
!MAMemento methodsFor: 'private' stamp: 'lr 2/8/2009 19:20'!
pull
	"Utility method to pull the model into a dictionary mapping descriptions to values. nil values are replaced with the default ones of the model."

	| result |
	result := self pullRaw.
	result keysAndValuesDo: [ :key :value |
		value isNil
			ifTrue: [ result at: key put: key default yourself ] ].
	^ result! !

!MAMemento methodsFor: 'private' stamp: 'lr 2/8/2009 19:20'!
push: aDictionary
	"Utility method to push a dictionary mapping descriptions to values into the model."

	aDictionary keysAndValuesDo: [ :key :value |
		(key isVisible and: [ key isReadonly not ]) 
			ifTrue: [ self model write: value using: key ] ]! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 2/5/2009 10:52'!
visitTokenDescription: aDescription
	| token |
	self object: (aDescription kind streamContents: [ :output |
		[ self stream atEnd ] whileFalse: [
			token := WriteStream on: String new.
			[ self stream atEnd or: [ aDescription separators includes: self stream peek ] ]
				whileFalse: [ token nextPut: self stream next ].
			self stream atEnd 
				ifFalse: [ self stream next ].
			token contents isEmpty
				ifFalse: [ output nextPut: (aDescription reference fromString: token contents) ] ] ])! !

!MANumberDescription methodsFor: 'convenience' stamp: 'lr 2/5/2009 21:16'!
beInteger
	self addCondition: (MACondition selector: #isInteger) labelled: 'No integer was entered'! !

!MANumberDescription methodsFor: 'convenience' stamp: 'lr 2/5/2009 21:16'!
beNegative
	self addCondition: (MACondition selector: #negative) labelled: 'No negative number was entered'! !

!MANumberDescription methodsFor: 'convenience' stamp: 'lr 2/5/2009 21:16'!
bePositive
	self addCondition: (MACondition selector: #positive) labelled: 'No positive number was entered'! !

!Object methodsFor: '*magritte-model-accessing' stamp: 'lr 2/18/2009 15:02'!
basicDescription
	"Return the description of the reciever. Subclasses might override this message to return instance-based descriptions."

	^ self class description! !

!Object methodsFor: '*magritte-model-accessing' stamp: 'lr 2/18/2009 15:05'!
description
	"Return the canonical description of the receiver. Never override this method, instead modify #description."

	^ self basicDescription! !

!Object methodsFor: '*magritte-model-model' stamp: 'lr 9/23/2008 08:15'!
initializeMagritte
	"Initializes the receiving object with the default values of its description."

	self description do: [ :each |
		each isReadonly
			ifFalse: [ self write: each default using: each ] ]! !

!MAValidationError methodsFor: 'testing' stamp: 'lr 9/21/2008 10:41'!
isResumable
	^ resumable ifNil: [ resumable := false ]! !

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 2/8/2009 19:20'!
openWorkspace: aContentsString titled: aTitleString
	"Open a new workspace with the contents aContentsString and the title aTitleString."

	Workspace new
		contents: aContentsString;
		openLabel: aTitleString! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 2/8/2009 19:20'!
referenceStream: aReadWriteStream
	"Return a stream instance to operate on aReadWriteStream being able to serialize and deserialize objects by sending #nextPut: and #next. Squeak: The implementation of ReferenceStream doesn't work well together with the default WriteStream implementation, therefor we have to change it on the fly."

	^ ReferenceStream on: (aReadWriteStream class = WriteStream
		ifTrue: [ RWBinaryOrTextStream on: aReadWriteStream contents ]
		ifFalse: [ aReadWriteStream ])! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 2/8/2009 19:20'!
uuid
	"Answer a random object that is extremely likely to be unique over space and time."

	^ UUID new! !

MAReferenceDescription subclass: #MATokenDescription
	instanceVariableNames: 'separators'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MATokenDescription commentStamp: 'lr 4/10/2007 21:11' prior: 0!
I am a description of tokens all described by the referenced description, possible values are instances of ==SequenceableCollection==.!

!MATokenDescription class methodsFor: 'accessing-defaults' stamp: 'lr 2/5/2009 10:56'!
defaultSeparators
	^ ' ,;' , String crlf , String tab! !

!MATokenDescription class methodsFor: 'accessing-description' stamp: 'lr 2/5/2009 10:45'!
descriptionSeparators
	^ MAStringDescription new
		accessor: #separators;
		label: 'Separators';
		priority: 250;
		yourself! !

!MATokenDescription methodsFor: 'accessing' stamp: 'lr 2/5/2009 10:46'!
separators
	^ separators ifNil: [ separators := self class defaultSeparators ]! !

!MATokenDescription methodsFor: 'accessing' stamp: 'lr 2/5/2009 10:57'!
separators: aString
	"A string with the separator characters to split tokens, the first character is used as a default for printing."

	separators := aString! !

!MAExternalFileModel class methodsFor: 'public' stamp: 'lr 9/9/2008 00:02'!
garbageCollect
	"Remove obsolete files from the file-system that do not have a counterpart in memory anymore. This method has to be called manually and is not intended to be portable."
	
	| mark sweep |
	mark := self allInstances
		collect: [ :each | each directory pathName ].
	sweep := Array with: self baseDirectory.
	self locationDefinition do: [ :definition |
		sweep := sweep gather: [ :directory |
			directory entries isEmpty
				ifTrue: [ directory recursiveDelete ].
			(directory directoryNames
				select: [ :each | each size = definition first ])
				collect: [ :each | directory directoryNamed: each ] ] ].
	sweep do: [ :directory |
		(mark includes: directory pathName)
			ifFalse: [ directory recursiveDelete ] ]! !

!MAExternalFileModel class methodsFor: 'initialization' stamp: 'lr 9/8/2008 23:26'!
locationDefinition
	^ #( ( 2 '63450af8d9c2e17b' ) ( 30 'iaojv41bw67e0tud5m9rgplqfy8x3cs2kznh' ) )! !

!MAExternalFileModel methodsFor: 'configuration' stamp: 'lr 9/8/2008 23:26'!
locationDefinition
	^ self class locationDefinition! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 2/5/2009 10:48'!
visitTokenDescription: aDescription
	self object
		do: [ :each | self object: each during: [ self visit: aDescription reference ] ]
		separatedBy: [ self stream nextPut: aDescription separators first ]! !

MAObject subclass: #MACondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MACondition class methodsFor: 'testing' stamp: 'lr 2/5/2009 21:05'!
isAbstract
	^ self name = #MACondition! !

!MACondition class methodsFor: 'convenience' stamp: 'lr 2/5/2009 21:30'!
receiver: anObject selector: aSelector
	^ self receiver: anObject selector: aSelector arguments: #(nil) index: 1! !

!MACondition class methodsFor: 'instance creation' stamp: 'lr 2/5/2009 21:30'!
receiver: anObject selector: aSelector arguments: anArray index: anInteger
	(aSelector numArgs = anArray size)
		ifFalse: [ self error: 'Invalid number of arguments.' ].
	(anInteger between: 0 and: anArray size)
		ifFalse: [ self error: 'Index out of bounds.' ].
	^ MAPluggableCondition new 
		initializeReceiver: anObject
		selector: aSelector
		arguments: anArray
		index: anInteger! !

!MACondition class methodsFor: 'convenience' stamp: 'lr 2/5/2009 21:30'!
selector: aSelector
	^ self receiver: nil selector: aSelector arguments: #() index: 0! !

!MACondition class methodsFor: 'convenience' stamp: 'lr 2/5/2009 21:30'!
selector: aSelector argument: anObject
	^ self receiver: nil selector: aSelector arguments: (Array with: anObject) index: 0! !

!MACondition methodsFor: 'operators' stamp: 'lr 2/5/2009 20:40'!
& aCondition 
	^ MAConjunctiveCondition new & self & aCondition! !

!MACondition methodsFor: 'operators' stamp: 'lr 2/5/2009 21:56'!
not
	^ MANegationCondition with: self! !

!MACondition methodsFor: 'accessing' stamp: 'lr 2/5/2009 20:56'!
numArgs
	^ 1! !

!MACondition methodsFor: 'evaluating' stamp: 'lr 2/5/2009 20:43'!
value: anObject
	self subclassResponsibility! !

!MACondition methodsFor: 'operators' stamp: 'lr 2/5/2009 20:40'!
| aCondition 
	^ (self not & aCondition not) not! !

MACondition subclass: #MAConjunctiveCondition
	instanceVariableNames: 'conditions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MAConjunctiveCondition methodsFor: 'operators' stamp: 'lr 2/5/2009 20:42'!
& aCondition
	conditions := conditions copyWith: aCondition! !

!MAConjunctiveCondition methodsFor: 'initialization' stamp: 'lr 2/5/2009 20:42'!
initialize
	conditions := #()! !

!MAConjunctiveCondition methodsFor: 'evaluating' stamp: 'lr 2/5/2009 20:44'!
value: anObject
	^ conditions allSatisfy: [ :each | each value: anObject ]! !

MACondition subclass: #MANegationCondition
	instanceVariableNames: 'condition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MANegationCondition class methodsFor: 'instance creation' stamp: 'lr 2/5/2009 20:45'!
with: aCondition
	^ self new initializeWith: aCondition! !

!MANegationCondition methodsFor: 'initialization' stamp: 'lr 2/5/2009 21:55'!
initializeWith: aCondition
	condition := aCondition! !

!MANegationCondition methodsFor: 'evaluating' stamp: 'lr 2/5/2009 20:44'!
value: anObject
	^ (condition value: anObject) not! !

MACondition subclass: #MAPluggableCondition
	instanceVariableNames: 'receiver selector arguments index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Utility'!

!MAPluggableCondition methodsFor: 'initialization' stamp: 'lr 2/5/2009 21:30'!
initializeReceiver: anObject selector: aSelector arguments: anArray index: anInteger
	receiver := anObject.
	selector := aSelector.
	arguments := anArray.
	index := anInteger! !

!MAPluggableCondition methodsFor: 'evaluating' stamp: 'lr 2/5/2009 21:31'!
value: anObject
	| actualReceiver actualArguments |
	actualReceiver := index = 0
		ifFalse: [ receiver ]
		ifTrue: [ anObject ].
	actualArguments := index > 0
		ifFalse: [ arguments ]
		ifTrue: [
			arguments copy 
				at: index put: anObject;
				yourself ].
	^ actualReceiver
		perform: selector
		withArguments: actualArguments! !

MAObject subclass: #MADescription
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MADescription commentStamp: 'lr 2/5/2009 12:42' prior: 0!
I am the root of the description hierarchy in Magritte and I provide most of the basic properties available to all descriptions. If you would like to annotate your model with a description have a look at the different subclasses of myself.

!!Example
If your model has an instance variable called ==title== that should be used to store the title of the object, you could add the following description to your class:

=Document class>>descriptionTitle
=	^ MAStringDescription new
=		autoAccessor: #title;
=		label: 'Title';
=		priority: 20;
=		beRequired;
=		yourself.

The selector ==#title== is the name of the accessor method used by Magritte to retrieve the value from the model. In the above case Magritte creates the accessor method and the instance variable automatically, if necessary. The label is used to give the field a name and will be printed next to the input box if a visual GUI is created from this description.

The write-accessor is automatically deduced by adding a colon to the read-selector, in this example ==#title:==. You can specify your own accessor strategy using one of the subclasses of ==*MAAccessor*==. If you have multiple description within the same object, the ==#priority:== field is used to order them. Assign a low priority to have descriptions traversed first.!

!MADescription methodsFor: 'validation' stamp: 'lr 2/5/2009 12:25'!
addCondition: aCondition labelled: aString
	"Add ==aCondition== as an additional validation condition to the receiver and give it the label ==aString==. The first argument is either a block-context or any other object that responds to ==#value:== with ==true== or ==false==."

	self conditions: (self conditions
		copyWith: (Association
			key: aCondition
			value: aString))! !

!MADescription methodsFor: 'accessors' stamp: 'lr 2/5/2009 12:30'!
autoAccessor: aSelector
	"Uses ==aSelector== to read from the model. Creates read and write accessors and instance-variables if necessary. This is very convenient for prototyping and can later be changed to a ==*selectorAccessor:>MADescription-selectorAccessor:*== using a simple rewrite rule."

	self accessor: (MAAutoSelectorAccessor selector: aSelector)! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 2/5/2009 12:24'!
conditions
	"Answer a collection of additional conditions that need to be fulfilled so that the described model is valid. Internally the collection associates conditions with an error string."

	^ self propertyAt: #conditions ifAbsent: [ self class defaultConditions ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 2/8/2009 19:20'!
group
	"Answer the group of the receiving description. The group is a string used to categorize and group descriptions. Certain display interpreters with be able to use this information to improve the usability."

	^ self propertyAt: #group ifAbsent: [ self class defaultGroup ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 2/8/2009 19:20'!
group: aString
	"Answer the group of the receiving description. The group is a string used to categorize and group descriptions. Certain display interpreters with be able to use this information to improve the usability."

	^ self propertyAt: #group put: aString! !
