
!MAMemento methodsFor: 'private' stamp: 'lr 3/9/2006 11:32'!
pull
	"Utitlity method to pull the model into a dictionary mapping descriptions to values. nil values are replaced with the default ones of the model."

	| result |
	result := self pullRaw.
	result keysAndValuesDo: [ :key :value |
		value isNil
			ifTrue: [ result at: key put: key default yourself ] ].
	^ result! !

!MAMemento methodsFor: 'private' stamp: 'lr 5/14/2008 20:31'!
push: aDictionary
	"Utitlity method to push a dictionary mapping descriptions to values into the model."

	aDictionary keysAndValuesDo: [ :key :value |
		(key isVisible and: [ key isReadonly not ]) 
			ifTrue: [ self model write: value using: key ] ]! !

!MAStringReader methodsFor: 'visiting-description' stamp: 'lr 3/9/2006 11:32'!
visitTokenDescription: aDescription
	self object: (aDescription kind streamContents: [ :output |
		[ self stream atEnd ] whileFalse: [
			output nextPut: (aDescription reference
				fromString: (self stream upTo: $ )) ] ])! !

!MANumberDescription methodsFor: 'conveniance' stamp: 'lr 10/8/2007 16:01'!
beInteger
	self addCondition: [ :value | value isInteger ] labelled: 'No integer was entered'! !

!Object methodsFor: '*magritte-model-accessing' stamp: 'lr 3/9/2006 11:31'!
description
	"Return the description of the reciever. Subclasses might override this message to return instance-based descriptions."

	^ self class description! !

!MAValidationError methodsFor: 'testing' stamp: 'lr 9/3/2008 14:33'!
isResumable
	^ resumable ifNil: [ false ]! !

!MACompatibility class methodsFor: 'user-interface' stamp: 'lr 3/9/2006 11:32'!
openWorkspace: aContentsString titled: aTitleString
	"Open a new wokspace with the contents aContentsString and the title aTitleString."

	Workspace new
		contents: aContentsString;
		openLabel: aTitleString! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 3/9/2006 11:32'!
referenceStream: aReadWriteStream
	"Return a stream instance to operate on aReadWriteStream being able to serialize and deserialize objects by sending #nextPut: and #next. Squeak: The implementation of ReferenceStream doesn't work well together with the default WriteStream implementaiton, therefor we have to change it on the fly."

	^ ReferenceStream on: (aReadWriteStream class = WriteStream
		ifTrue: [ RWBinaryOrTextStream on: aReadWriteStream contents ]
		ifFalse: [ aReadWriteStream ])! !

!MACompatibility class methodsFor: 'environment' stamp: 'lr 3/9/2006 11:32'!
uuid
	"Answer a random object that is extremly likely to be unique over space and time."

	^ UUID new! !

MAObject subclass: #MADescription
	instanceVariableNames: 'accessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MADescription commentStamp: 'lr 4/18/2007 17:47' prior: 0!
I am the root of the description hierarchy in Magritte and I provide most of the basic properties available to all descriptions. If you would like to annotate your model with a description have a look at the different subclasses of myself.

!!Example
If your model has an instance variable called ==title== that should be used to store the title of the object, you could add the following description to your class:

=Document class>>descriptionTitle
=	^ MAStringDescription new
=		autoAccessor: #title;
=		label: 'Title';
=		priority: 20;
=		beRequired;
=		yourself.

The selector ==#title== is the name of the accessor method used by Magritte to retrieve the value from the model. In the above case Magritte creates the accessor method and the instance variable automatically, if necessary. The label is used to give the field a name and will be printed next to the input box if a visual GUI is created from this description.

The write-accessor is automatically deduced by adding a colon to the read-selector, in this example ==#title:==. You can specify your own accessor strategy using one of the subclasses of ==*MAAccessor*==. If you have multiple description within the same object, the ==#priority:# field is used to order them. Assign a low priority to have descriptions traversed first.!

!MADescription methodsFor: 'validation' stamp: 'lr 4/10/2007 21:33'!
addCondition: aCondition labelled: aString
	"Add ==aCondition== as an additional validation condition to the receiver and give it the label ==aString==. The first argument is either a block-context, a composite of the subclasses of ==*MACondition*==, or any other object that responds to ==#value:== with ==true== or ==false==."

	self conditions: (self conditions
		copyWith: (Association
			key: aCondition
			value: aString))! !

!MADescription methodsFor: 'accessors' stamp: 'lr 9/12/2007 17:26'!
autoAccessor: aSelector
	"Uses ==aSelector== to read from the model. Creates read and write accessors and instance-variables if necessary. This is very conveniant for prototyping and can later be changed to a ==*selectorAccessor:*== using a simple rewrite rule."

	self accessor: (MAAutoSelectorAccessor selector: aSelector)! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 3/9/2006 11:32'!
conditions
	"Answer a collection of additional conditions that need to be fulfilled so that the described model is valid. Internally the collection associates conditions, that are either blocks or subclasses of *MACondition*, with an error string."

	^ self propertyAt: #conditions ifAbsent: [ self class defaultConditions ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 10/27/2007 12:00'!
group
	"Answer the group of the receiving description. The group is a string used to categorize and group descriptions. Certain display interpreters with be able to use this information to improve the useability."

	^ self propertyAt: #group ifAbsent: [ self class defaultGroup ]! !

!MADescription methodsFor: 'accessing-properties' stamp: 'lr 10/27/2007 12:00'!
group: aString
	"Answer the group of the receiving description. The group is a string used to categorize and group descriptions. Certain display interpreters with be able to use this information to improve the useability."

	^ self propertyAt: #group put: aString! !

!MAStringWriter methodsFor: 'visiting-description' stamp: 'lr 3/9/2007 14:19'!
visitTokenDescription: aDescription
	self object
		do: [ :each | self object: each during: [ self visit: aDescription reference ] ]
		separatedBy: [ self stream nextPutAll: ' ' ]! !

MAReferenceDescription subclass: #MATokenDescription
	instanceVariableNames: 'tokens'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-Model-Description'!

!MATokenDescription commentStamp: 'lr 4/10/2007 21:11' prior: 0!
I am a description of tokens all described by the referenced description, possible values are instances of ==SequenceableCollection==.!

!MATokenDescription methodsFor: 'accessing' stamp: 'lr 3/9/2007 14:28'!
tokens
	^ tokens ifNil: [ tokens := #() ]! !

!MATokenDescription methodsFor: 'accessing' stamp: 'lr 3/9/2007 14:28'!
tokens: anArray
	tokens := anArray! !

!MAExternalFileModel methodsFor: 'configuration' stamp: 'lr 1/26/2007 18:23'!
locationDefinition
	^ #( ( 2 '63450af8d9c2e17b' ) ( 30 'iaojv41bw67e0tud5m9rgplqfy8x3cs2kznh' ) )! !
